<?php

/*

implements macron vowels using iso-latin-2 entities from the DocBookDTD
by mapping the entities to the appropriate unicode character

also enhances link text handling to support wiki markup

Copyright 2003, 2004 John Rankin john.rankin@affinity.co.nz

*/
$FmtWikiLinkPreEntities = $FmtWikiLink;
$InlineLinkReplacements = array(
  "/'''''(.*?)'''''/" => "<em><strong>\$1</strong></em>",
  "/'''(.*?)'''/" => "<strong>\$1</strong>",
  "/''(.*?)''/" => "<em>\$1</em>",
  "/@@(.*?)@@/" => "<code>\$1</code>",
  "/\\[(\\++)(.*?)\\1\\]/e" =>
    "'<span style=\'font-size:'.(round(pow(1.2,strlen('$1'))*100,0)).'%\'>'.str_replace('\\\"','\"','$2').'</span>'",
  "/\\[(-+)(.*?)\\1\\]/e" =>
    "'<span style=\'font-size:'.(round(pow(5/6,strlen('$1'))*100,0)).'%\'>'.str_replace('\\\"','\"','$2').'</span>'",
  "/&amp;([A-Za-z0-9]+;|#\d+;)/" => "&\$1",
  "/\\^\\^(.*?)\\^\\^/" => "<sup>$1</sup>",
  "/__(.*?)__/" => "<sub>$1</sub>",
  "/\\[(.*?)\\|(.*?)\\]/" => "<acronym title='$2'>$1</acronym >",
  "/--/" => "&mdash;",
  "/`-/" => "&ndash;",
  "/\.\.\./" => "&hellip;",
  "/(<.*?>['\"]*)|((.?)(['\"]+))/e" => 'BypassHTML("$1","$2")',
);

$Vowels = array (
    "/((&amp;)|{)A((m(acr)?;)|})/" => "&#256;",
    "/((&amp;)|{)a((m(acr)?;)|})/" => "&#257;",
    "/((&amp;)|{)E((m(acr)?;)|})/" => "&#274;",
    "/((&amp;)|{)e((m(acr)?;)|})/" => "&#275;",
    "/((&amp;)|{)I((m(acr)?;)|})/" => "&#298;",
    "/((&amp;)|{)i((m(acr)?;)|})/" => "&#299;",
    "/((&amp;)|{)O((m(acr)?;)|})/" => "&#332;",
    "/((&amp;)|{)o((m(acr)?;)|})/" => "&#333;",
    "/((&amp;)|{)U((m(acr)?;)|})/" => "&#362;",
    "/((&amp;)|{)u((m(acr)?;)|})/" => "&#363;",
    "/&amp;f(rac)?14;/" => "&#188;",
    "/&amp;f(rac)?12;/" => "&#189;",
    "/&amp;f(rac)?34;/" => "&#190;",
);

$InlineReplacements = array_merge_recursive ($Vowels, $InlineReplacements);
$InlineLinkReplacements = 
	array_merge_recursive ($Vowels, $InlineLinkReplacements);
$FmtUrlLink  = 'FmtUrlLinkEntities';
$FmtWikiLink = 'FmtWikiLinkEntities';
$WikiPageSelfRefFmt = "<b class='selfreference'>\$LinkText</b>";
$WikiPageExistsFragFmt = 
    "<a class='wikiword' href='\$PageUrl\$Fragment'>\$LinkText</a>";
$WikiPageExistsTitleFmt = 
    "<a class='wikiword' href='\$PageUrl' title=\$ToolTip>\$LinkText</a>";
if ($action=='browse') {
    $AltTitle = TitleParagraph($pagename);
    if ($AltTitle) $HTMLTitleFmt = 
                        str_replace('$HTMLTitle',$AltTitle,$HTMLTitleFmt);
}

$FmtThisUrlLink = 'FmtThisUrlLink';
$ThisUrlLinkFmt = "<a class='thisurl' href='\$Url'>\$LinkText</a>";

function FmtThisUrlLink($pat,$ref,$btext) {
  global $ThisUrlLinkFmt,$UrlLinkTextFmt;
  $h=$UrlLinkTextFmt;
  $UrlLinkTextFmt=$ThisUrlLinkFmt;
  $r= FmtUrlLinkEntities($pat,$ref,$btext);
  $UrlLinkTextFmt=$h;
  return $r;
}

function FmtUrlLinkEntities($pat,$ref,$btext) {
  if (!is_null($btext)) { $btext = FmtTextReplacements($btext); }
  return FmtUrlLink($pat,$ref,$btext);
}

function FmtWikiLinkEntities($pat,$ref,$btext,$out=NULL,$pname=NULL) {
  global $WikiPageSelfRefFmt,$WikiPageExistsFmt,$SpaceWikiWords,
         $WikiWordPattern,$AnyLetter,$WikiPageExistsFragFmt,
         $WikiPageExistsTitleFmt,$AsSpacedFunction,$DefaultTitle,
         $FmtWikiLinkPreEntities;
  if (!is_null($btext)) { $btext = FmtTextReplacements($btext); }
  if (preg_match("/{{($WikiWordPattern)(\\|$AnyLetter+)?}}/",$ref,$match)) {
      if ($SpaceWikiWords) $ref=str_replace($match[1],
                                            $AsSpacedFunction($match[1]),$ref);
  }
  $h=$WikiPageExistsFmt;
  $pg = FmtWikiLink($pat,$ref,$btext,'PageName',$pname);
  if ($GLOBALS['pagename']==$pg) {
      $WikiPageExistsFmt=$WikiPageSelfRefFmt;
      if (strstr($ref,'#'))  $WikiPageExistsFmt=$WikiPageExistsFragFmt;
  } elseif (!PageExists($pg)) {
      $t = preg_replace("/^.*\\./",'',$pg);
      $tref = ($SpaceWikiWords) ? $AsSpacedFunction($t) : $t;
      if (PageExists("$t.$t")) $ref = $t . "/{{" . $tref . "}}";
      elseif (PageExists("$t.$DefaultTitle")) {
        $ref = "$t/$DefaultTitle";
        if ($btext=='') $btext = $tref;
      }
  } elseif (strstr($ref,'#')) {
      $WikiPageExistsFmt=$WikiPageExistsFragFmt;
  } else {
      $title = TitleParagraph($pg);
      if ($title) { 
          $title = (strstr($title,"'")) ? '"'.$title.'"' : "'$title'";
          $WikiPageExistsFmt = 
                      str_replace('$ToolTip',$title,$WikiPageExistsTitleFmt);
      }
  }
  $r = $FmtWikiLinkPreEntities($pat,$ref,$btext,$out,$pname);
  $WikiPageExistsFmt=$h;
  return $r;
}

function FmtTextReplacements($linktext) {
  global $InlineLinkReplacements;
  foreach ($InlineLinkReplacements as $pat => $rep) 
        $linktext = preg_replace($pat,$rep,$linktext);
  return $linktext;
}

function TitleParagraph($page) {
  $refpage = ReadPage($page);
  if ($refpage) {
    $refpage['text'] = $refpage['text'] . "\n";
    if (preg_match("/^(?:!+|:.*?:)\\s*(?:\\[\\[#.*?\\]\\])?(.*?)\\n/", $refpage['text'], $match)) {
        $para = preg_replace("/''+(.*?)''+/",'$1',$match[1]);
        $para = preg_replace("/\\[.*?$/","...",$para);
        $para = preg_replace("/`\\..*?$/","...",$para);
        $para = str_replace('"','&quot;',
                str_replace('`','',str_replace('`-','&ndash;',$para)));
    }
  }
  return $para;
}

?>
